"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const app_json_file_1 = require("../util/app-json-file");
const constants_1 = require("../constants");
const confirm_write_file_prompt_component_1 = require("./confirm-write-file-prompt-component");
const find_or_write_app_py_file_component_1 = require("./find-or-write-app-py-file-component");
const alwayscli_1 = require("@alwaysai/alwayscli");
const unable_to_proceed_without_message_1 = require("../util/unable-to-proceed-without-message");
const spinner_1 = require("../util/spinner");
const check_for_project_id_1 = require("../components/projects-and-teams/check-for-project-id");
const js_spawner_1 = require("../util/spawner/js-spawner");
const WRITE_MESSAGE = `Write ${constants_1.APP_JSON_FILE_NAME}`;
const FOUND_MESSAGE = `Found ${constants_1.APP_JSON_FILE_NAME}`;
async function findOrWriteAppJsonFileComponent(props) {
    const { yes, use } = props;
    const appJsonFile = app_json_file_1.AppJsonFile();
    if (appJsonFile.exists()) {
        // if it exits check if it has a project id
        if (appJsonFile.read().project) {
            const contributor = await check_for_project_id_1.checkForProjectId();
            if (!contributor) {
                throw new alwayscli_1.CliTerseError('You are not part of the team. Make sure to add the project owner to add you.');
            }
            // if this happens dont init the remote step
            // check that the project id belongs to user
        }
        else {
            console.log('No project id found in app.json');
            const selectedProject = await check_for_project_id_1.selectFromListOfProjects();
            appJsonFile.addProjectId(selectedProject['id'].toString());
            // do this part first
            // case where the app id does not exist in a json fille
            // all that needs to be done is add a project id
        }
    }
    else {
        let selectedProject;
        if (use) {
            const listOfProjects = await check_for_project_id_1.getListOfProjects();
            const hasProject = listOfProjects.filter(project => project['name'] === use);
            if (hasProject.length) {
                selectedProject = hasProject[0];
            }
            else {
                selectedProject = await check_for_project_id_1.selectFromListOfProjects();
            }
        }
        else {
            selectedProject = await check_for_project_id_1.selectFromListOfProjects();
        }
        // pick which project type you need
        // for this example choose blank
        // blank will keep same flow
        // starter apps already have own json so no need to create
        // so just divide these steps into another if and else
        // read the project and check what kind of starter app it has if its empty then blank
        if (selectedProject.starter_app) {
            console.log(selectedProject.starter_app);
            await check_for_project_id_1.downloadSingleApp(process.cwd(), selectedProject.starter_app);
            appJsonFile.addProjectId(selectedProject['id'].toString());
        }
        else if (selectedProject.git_url) {
            const spawner = js_spawner_1.JsSpawner();
            spawner.runForegroundSync({
                exe: 'git',
                args: ['clone', selectedProject.git_url],
            });
        }
        else {
            const projectType = await check_for_project_id_1.selectTypeOfProject();
            if (projectType.projects === 'Start Blank') {
                const confirmed = yes ||
                    (await confirm_write_file_prompt_component_1.confirmWriteFilePromptComponent({
                        fileName: constants_1.APP_JSON_FILE_NAME,
                        description: 'Configuration file',
                    }));
                if (!confirmed) {
                    throw new alwayscli_1.CliTerseError(unable_to_proceed_without_message_1.UnableToProceedWithoutMessage(constants_1.APP_JSON_FILE_NAME));
                }
                try {
                    appJsonFile.initialize();
                    appJsonFile.addProjectId(selectedProject['id'].toString());
                    spinner_1.Spinner(WRITE_MESSAGE).succeed();
                }
                catch (exception) {
                    spinner_1.Spinner(WRITE_MESSAGE).fail();
                    throw exception;
                }
                await find_or_write_app_py_file_component_1.findOrWriteAppPyFileComponent({ yes });
            }
            else if (projectType.projects === 'Use starter app') {
                const pickedStarterApp = await check_for_project_id_1.readJson();
                await check_for_project_id_1.downloadSingleApp(process.cwd(), pickedStarterApp);
                appJsonFile.addProjectId(selectedProject['id'].toString());
            }
        }
    }
}
exports.findOrWriteAppJsonFileComponent = findOrWriteAppJsonFileComponent;
//# sourceMappingURL=find-or-write-app-json-file-component.js.map