"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rpc_client_1 = require("../../util/rpc-client");
const authentication_client_1 = require("../../util/authentication-client");
const app_json_file_1 = require("../../util/app-json-file");
const system_id_1 = require("../../util/system-id");
const inquirer = require("inquirer");
const js_spawner_1 = require("../../util/spawner/js-spawner");
const cloud_api_1 = require("@alwaysai/cloud-api");
const node_fetch_1 = require("node-fetch");
const cd_1 = require("../../util/cd");
const node_machine_id_1 = require("node-machine-id");
const os = require('os');
const target_json_file_1 = require("../../util/target-json-file");
const util = require('util');
const fs = require('fs');
const streamPipeline = util.promisify(require('stream').pipeline);
const alwayscli_1 = require("@alwaysai/alwayscli");
const path_1 = require("path");
const rimraf = require("rimraf");
const mkdirp = require("mkdirp");
const constants_1 = require("../../constants");
const fs_1 = require("fs");
const tar = require("tar");
const pump = require("pump");
const app_configure_preliminary_steps_component_1 = require("../app-configure-preliminary-steps-component");
async function checkForProjectId() {
    const username = await authentication_client_1.CliAuthenticationClient().getInfo();
    const listProjectsUserIsCollaborator = await rpc_client_1.CliRpcClient().listProjectsUserIsCollaborator({
        user_name: username.username,
    });
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const collabProject = listProjectsUserIsCollaborator.filter(value => {
            if (project.projectId === value.id.toString()) {
                return value.id.toString();
            }
        });
        if (!collabProject.length) {
            return false;
        }
        return collabProject;
    }
}
exports.checkForProjectId = checkForProjectId;
async function selectFromListOfProjects() {
    const username = await authentication_client_1.CliAuthenticationClient().getInfo();
    const listProjectsUserIsCollaborator = await rpc_client_1.CliRpcClient().listProjectsUserIsCollaborator({
        user_name: username.username,
    });
    // check if listNames is not empty
    const questions = [
        {
            type: 'list',
            name: 'projects',
            message: 'Select from list of projects or create a new project at (https://dashboard.a6i1.net/dashboard/create-team):',
            choices: listProjectsUserIsCollaborator,
            filter(val) {
                return val.toLowerCase();
            },
        },
    ];
    return await inquirer.prompt(questions).then(answer => {
        const pickedProject = listProjectsUserIsCollaborator.filter(value => {
            if (value.name.toLocaleLowerCase() === answer.projects) {
                return value;
            }
        });
        return pickedProject[0];
    });
}
exports.selectFromListOfProjects = selectFromListOfProjects;
async function getListOfProjects() {
    const username = await authentication_client_1.CliAuthenticationClient().getInfo();
    const listProjectsUserIsCollaborator = await rpc_client_1.CliRpcClient().listProjectsUserIsCollaborator({
        user_name: username.username,
    });
    return listProjectsUserIsCollaborator;
}
exports.getListOfProjects = getListOfProjects;
async function selectTypeOfProject() {
    const questions = [
        {
            type: 'list',
            name: 'projects',
            message: 'Select Project source',
            choices: ['Use starter app', 'Start Blank'],
        },
    ];
    return await inquirer.prompt(questions).then(answer => {
        return answer;
    });
}
exports.selectTypeOfProject = selectTypeOfProject;
async function localProject() {
    // check if project has a git url`
    const spawner = js_spawner_1.JsSpawner();
    spawner.runForegroundSync({
        exe: 'git',
        args: ['clone', 'https://github.com/alwaysai/alwaysai-twilio-sms'],
    });
}
exports.localProject = localProject;
async function readJson() {
    const systemDomainName = cloud_api_1.SystemDomainName(system_id_1.getSystemId());
    const starterAppsUrl = `https://dashboard.${systemDomainName}/docs/apps-test-json.json`;
    const authorizationHeader = await authentication_client_1.CliAuthenticationClient().getAuthorizationHeader();
    const jwt = authorizationHeader.Authorization.split(' ')[1];
    const cookieHeader = {
        cookie: `token=${jwt}`,
    };
    try {
        const response = await node_fetch_1.default(starterAppsUrl, {
            redirect: 'manual',
            headers: Object.assign({}, cookieHeader),
        });
        const readerStream = await response.buffer();
        const parsedJSon = JSON.parse(readerStream.toString());
        const questions = [
            {
                type: 'list',
                name: 'starterApps',
                message: 'Select a starter app):',
                choices: parsedJSon['starter-apps'],
                filter(val) {
                    return val.toLowerCase();
                },
            },
        ];
        return await inquirer.prompt(questions).then(answer => {
            return answer['starterApps'];
        });
    }
    finally {
    }
}
exports.readJson = readJson;
async function downloadSingleApp(dir = process.cwd(), pickedStarterApp) {
    const systemDomainName = cloud_api_1.SystemDomainName(system_id_1.getSystemId());
    const starterAppsUrl = `https://dashboard.${systemDomainName}/docs/${pickedStarterApp}.tar.gz`;
    const destinationDir = path_1.join(dir, pickedStarterApp);
    if (fs_1.existsSync(destinationDir)) {
        throw new alwayscli_1.CliTerseError(`Directory "${destinationDir}" already exists`);
    }
    const tmpDir = `${destinationDir}.download`;
    rimraf.sync(tmpDir);
    const authorizationHeader = await authentication_client_1.CliAuthenticationClient().getAuthorizationHeader();
    const jwt = authorizationHeader.Authorization.split(' ')[1];
    const cookieHeader = {
        cookie: `token=${jwt}`,
    };
    mkdirp.sync(tmpDir);
    try {
        const response = await node_fetch_1.default(starterAppsUrl, {
            redirect: 'manual',
            headers: Object.assign({}, cookieHeader),
        });
        if (!response.ok) {
            throw new alwayscli_1.CliTerseError(`Server responded ${response.status} "${response.statusText}"`);
        }
        await new Promise((resolve, reject) => {
            pump([response.body, tar.extract({ cwd: tmpDir })], err => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        });
        const fileNames = fs_1.readdirSync(tmpDir);
        if (fileNames.length !== 1 || !fileNames[0]) {
            throw new Error(`Expected ${constants_1.ALWAYSAI_STARTER_APPS} package to contain a single directory. ${constants_1.PLEASE_REPORT_THIS_ERROR_MESSAGE}`);
        }
        await fs_1.renameSync(path_1.join(tmpDir, fileNames[0]), destinationDir);
        const starterAppFiles = fs_1.readdirSync(destinationDir);
        starterAppFiles.forEach(file => {
            fs_1.renameSync(path_1.join(destinationDir, file), path_1.join(dir, file));
        });
    }
    finally {
        rimraf.sync(tmpDir);
        rimraf.sync(destinationDir);
    }
}
exports.downloadSingleApp = downloadSingleApp;
async function newConfig() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    if (appJsonFile.exists()) {
        await checkForProjectId();
    }
    else {
        const selectedProject = await selectFromListOfProjects();
        // check if it has a git url
        if (selectedProject.git_url) {
            console.log('this is the git url process');
            const spawner = js_spawner_1.JsSpawner();
            spawner.runForegroundSync({
                exe: 'git',
                args: ['clone', selectedProject.git_url],
            });
        }
        else {
            const projectSource = await selectTypeOfProject();
            if (projectSource.projects === 'Use starter app') {
                const pickedStarterApp = await readJson();
                await downloadSingleApp(process.cwd(), pickedStarterApp);
                cd_1.cd(`${process.cwd()}/${pickedStarterApp}`);
                const newJson = app_json_file_1.AppJsonFile();
                newJson.addProjectId(selectedProject['id'].toString());
            }
            else {
                const yes = false;
                await app_configure_preliminary_steps_component_1.appConfigurePreliminaryStepsComponent({ yes });
                console.log('create blank project');
                appJsonFile.addProjectId(selectedProject['id'].toString());
            }
        }
    }
}
exports.newConfig = newConfig;
async function configFlow() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    if (appJsonFile.exists()) {
        const selectedProject = await selectFromListOfProjects();
        // check if it has a git url
        if (selectedProject.git_url) {
            console.log('this is the git url process');
            const spawner = js_spawner_1.JsSpawner();
            spawner.runForegroundSync({
                exe: 'git',
                args: ['clone', selectedProject.git_url],
            });
        }
        else {
            const projectSource = await selectTypeOfProject();
            if (projectSource.projects === 'Use starter app') {
                const pickedStarterApp = await readJson();
                await downloadSingleApp(process.cwd(), pickedStarterApp);
                cd_1.cd(`${process.cwd()}/${pickedStarterApp}`);
                const newJson = app_json_file_1.AppJsonFile();
                newJson.addProjectId(selectedProject['id'].toString());
            }
            else {
                const yes = false;
                console.log('create blank project');
                appJsonFile.addProjectId(selectedProject['id'].toString());
            }
        }
    }
}
exports.configFlow = configFlow;
async function initWithArgs() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    if (appJsonFile.exists()) {
        await checkForProjectId();
    }
    else {
        const selectedProject = await selectFromListOfProjects();
        // check if it has a git url
        if (selectedProject.git_url) {
            console.log('this is the git url process');
            const spawner = js_spawner_1.JsSpawner();
            spawner.runForegroundSync({
                exe: 'git',
                args: ['clone', selectedProject.git_url],
            });
        }
        else {
            const projectSource = await selectTypeOfProject();
            if (projectSource.projects === 'Use starter app') {
                const pickedStarterApp = await readJson();
                await downloadSingleApp(process.cwd(), pickedStarterApp);
                cd_1.cd(`${process.cwd()}/${pickedStarterApp}`);
                const newJson = app_json_file_1.AppJsonFile();
                newJson.addProjectId(selectedProject['id'].toString());
            }
            else {
                const yes = false;
                await app_configure_preliminary_steps_component_1.appConfigurePreliminaryStepsComponent({ yes });
                console.log('create blank project');
                appJsonFile.addProjectId(selectedProject['id'].toString());
            }
        }
    }
}
exports.initWithArgs = initWithArgs;
async function deviceFriendlyName() {
    const questions = [
        {
            type: 'input',
            name: 'deviceFriendlyName',
            message: 'Enter a device friendly name:',
        },
    ];
    return await inquirer.prompt(questions).then(answer => {
        console.log({ answer });
        return answer.deviceFriendlyName;
    });
}
async function checkIfDeviceHasBeenRegistered() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const projectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        const machineId = node_machine_id_1.machineIdSync();
        const newProjectDevices = projectDevices.filter(value => {
            if (value.hardware_ids === machineId) {
                return true;
            }
        });
        if (newProjectDevices[0]) {
            return true;
        }
    }
    return false;
}
exports.checkIfDeviceHasBeenRegistered = checkIfDeviceHasBeenRegistered;
async function sshCheckIfDeviceHasBeenRegistered(machineId) {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const projectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        const newProjectDevices = projectDevices.filter(value => {
            if (value.hardware_ids === machineId) {
                return true;
            }
        });
        if (newProjectDevices[0]) {
            return true;
        }
    }
    return false;
}
async function storeDeviceId() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    const checkDeviceLimit = await checkTeamLimit();
    if (checkDeviceLimit) {
        const inputDeviceFriendlyName = await deviceFriendlyName();
        const machineId = node_machine_id_1.machineIdSync();
        const username = await authentication_client_1.CliAuthenticationClient().getInfo();
        const hostname = os.hostname();
        const deviceUserName = os.userInfo().username;
        // check if device has already been registered
        const isDeviceRegistered = await checkIfDeviceHasBeenRegistered();
        if (!isDeviceRegistered) {
            const createDevice = await rpc_client_1.CliRpcClient().createDevice({
                owner: username.username,
                friendly_name: inputDeviceFriendlyName,
                host_name: hostname,
                device_user_name: deviceUserName,
                description: '',
                hardware_ids: machineId,
                device_hash: '',
            });
            const appJsonFile = app_json_file_1.AppJsonFile();
            const project = appJsonFile.read().project;
            if (project) {
                const addDeviceToProject = await rpc_client_1.CliRpcClient().addProjectDevice({
                    device_id: createDevice.id,
                    project_id: Number(project.projectId),
                });
                await saveDeviceJson(createDevice.id.toString());
            }
        }
        else {
            throw new alwayscli_1.CliTerseError('You already registered this device please pick a saved device');
        }
    }
    else {
        throw new alwayscli_1.CliTerseError('You are at your limit');
    }
}
exports.storeDeviceId = storeDeviceId;
async function sshStoreDeviceId(machineId) {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    const targetJsonFile = target_json_file_1.TargetJsonFile();
    const targetHostSpawner = targetJsonFile.readHostSpawner();
    const checkDeviceLimit = await checkTeamLimit();
    const hostname = await targetHostSpawner.run({
        exe: 'hostname',
    });
    const deviceUserName = await targetHostSpawner.run({
        exe: 'whoami',
    });
    if (checkDeviceLimit) {
        const inputDeviceFriendlyName = await deviceFriendlyName();
        const username = await authentication_client_1.CliAuthenticationClient().getInfo();
        // check if device has already been registered
        const isDeviceRegistered = await sshCheckIfDeviceHasBeenRegistered(machineId);
        if (!isDeviceRegistered) {
            const createDevice = await rpc_client_1.CliRpcClient().createDevice({
                owner: username.username,
                friendly_name: inputDeviceFriendlyName,
                host_name: hostname,
                device_user_name: deviceUserName,
                description: '',
                hardware_ids: machineId,
                device_hash: '',
            });
            const appJsonFile = app_json_file_1.AppJsonFile();
            const project = appJsonFile.read().project;
            if (project) {
                const addDeviceToProject = await rpc_client_1.CliRpcClient().addProjectDevice({
                    device_id: createDevice.id,
                    project_id: Number(project.projectId),
                });
                await saveDeviceJson(createDevice.id.toString());
            }
        }
        else {
            throw new alwayscli_1.CliTerseError('You already registered this device please pick a saved device in this remote device');
        }
    }
    else {
        throw new alwayscli_1.CliTerseError('You are at your limit');
    }
}
async function checkTeamLimit() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const teamId = await rpc_client_1.CliRpcClient().getProject({ id: Number(project.projectId) });
        const getDeviceLimits = await rpc_client_1.CliRpcClient().checkTeamLimits({
            team_id: teamId.team_id,
        });
        const devicesLimit = getDeviceLimits['devices'].limit;
        const devicesUsed = getDeviceLimits['devices'].used;
        if (devicesLimit > devicesUsed) {
            return true;
        }
    }
    return false;
}
exports.checkTeamLimit = checkTeamLimit;
async function checkIfDeviceIsBeingUsed(hardwareIds) {
    const machineId = node_machine_id_1.machineIdSync();
    if (machineId === hardwareIds) {
        console.log(true);
        return true;
    }
    return false;
}
exports.checkIfDeviceIsBeingUsed = checkIfDeviceIsBeingUsed;
async function sshCheckIfDeviceIsBeingUsed(hardwareIds, machineId) {
    console.log({ hardwareIds });
    console.log({ machineId });
    if (machineId === hardwareIds) {
        return true;
    }
    return false;
}
async function saveDeviceJson(id) {
    // saves device id in json
    const appJsonFile = app_json_file_1.AppJsonFile();
    appJsonFile.addDeviceId(id);
}
exports.saveDeviceJson = saveDeviceJson;
async function createNewOrPick() {
    const noSaved = await checkIfAnySavedDevices();
    let choices = [];
    if (noSaved) {
        choices = ['Use saved device', 'Create new device'];
    }
    else {
        choices = ['Create new device'];
    }
    const questions = [
        {
            type: 'list',
            name: 'deviceChoice',
            message: 'Select choice:',
            choices,
        },
    ];
    return await inquirer.prompt(questions).then(answer => {
        return answer.deviceChoice;
    });
}
exports.createNewOrPick = createNewOrPick;
async function sshCreateNewOrPick(machineId) {
    const noSaved = await sshCheckIfAnySavedDevices(machineId);
    let choices = [];
    if (noSaved) {
        choices = ['Use saved device', 'Create new device'];
    }
    else {
        choices = ['Create new device'];
    }
    const questions = [
        {
            type: 'list',
            name: 'deviceChoice',
            message: 'Select choice:',
            choices,
        },
    ];
    return await inquirer.prompt(questions).then(answer => {
        return answer.deviceChoice;
    });
}
async function checkIfAnySavedDevices() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const projectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        if (projectDevices.length) {
            return true;
        }
    }
    return false;
}
exports.checkIfAnySavedDevices = checkIfAnySavedDevices;
async function sshCheckIfAnySavedDevices(machineId) {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const projectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        if (projectDevices.length) {
            return true;
        }
    }
    return false;
}
async function pickDevice() {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    if (project) {
        const projectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        const newObject = [];
        projectDevices.forEach(value => {
            if (value) {
                newObject.push({
                    name: value.friendly_name,
                    value: { id: value.id, hardwareIds: value.hardware_ids },
                });
            }
        });
        const questions = [
            {
                type: 'list',
                name: 'device',
                message: 'Select a device:',
                choices: newObject,
            },
        ];
        return await inquirer.prompt(questions).then(answer => {
            return answer.device;
        });
    }
    // check json file if there is a device id present
}
exports.pickDevice = pickDevice;
async function pickAndStoreDevice() {
    const pickedDevice = await pickDevice();
    // reference device to current device
    const canIUse = await checkIfDeviceIsBeingUsed(pickedDevice.hardwareIds);
    if (canIUse) {
        await saveDeviceJson(pickedDevice.id.toString());
    }
    else {
        console.log('pick another device');
    }
}
exports.pickAndStoreDevice = pickAndStoreDevice;
async function sshPickAndStoreDevice(machineId) {
    const pickedDevice = await pickDevice();
    // reference device to current device
    const canIUse = await sshCheckIfDeviceIsBeingUsed(pickedDevice.hardwareIds, machineId);
    if (canIUse) {
        await saveDeviceJson(pickedDevice.id.toString());
    }
    else {
        throw new alwayscli_1.CliTerseError('The device picked does not match selected device. Please pick another one or make a new one.');
    }
}
async function deviceFlow() {
    const firstCheck = await checkIfDeviceIsStored();
    if (!firstCheck) {
        const deviceSelection = await createNewOrPick();
        if (deviceSelection === 'Use saved device') {
            await pickAndStoreDevice();
        }
        else {
            await storeDeviceId();
        }
    }
}
exports.deviceFlow = deviceFlow;
async function sshDeviceFlow(machineId) {
    const firstCheck = await sshCheckIfDeviceIsStored(machineId);
    if (!firstCheck) {
        const deviceSelection = await sshCreateNewOrPick(machineId);
        if (deviceSelection === 'Use saved device') {
            await sshPickAndStoreDevice(machineId);
        }
        else {
            await sshStoreDeviceId(machineId);
        }
    }
}
exports.sshDeviceFlow = sshDeviceFlow;
async function checkIfDeviceIsStored() {
    const machineId = node_machine_id_1.machineIdSync();
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    const device = appJsonFile.read().device;
    if (project && device) {
        const getProjectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        // need to validate with json file also
        const validDevice = getProjectDevices.filter(value => {
            if (value.id === Number(device.deviceId)) {
                return value;
            }
        });
        if (validDevice[0]) {
            if (validDevice[0].hardware_ids === machineId) {
                return true;
            }
        }
    }
    return false;
}
exports.checkIfDeviceIsStored = checkIfDeviceIsStored;
async function sshCheckIfDeviceIsStored(machineId) {
    const appJsonFile = app_json_file_1.AppJsonFile();
    const project = appJsonFile.read().project;
    const device = appJsonFile.read().device;
    if (project && device) {
        const getProjectDevices = await rpc_client_1.CliRpcClient().getProjectDevices({
            project_id: Number(project.projectId),
        });
        // need to validate with json file also
        const validDevice = getProjectDevices.filter(value => {
            if (value.id === Number(device.deviceId)) {
                return value;
            }
        });
        if (validDevice[0]) {
            if (validDevice[0].hardware_ids === machineId) {
                return true;
            }
        }
    }
    return false;
}
async function checkIfUserBelongsToProject() { }
exports.checkIfUserBelongsToProject = checkIfUserBelongsToProject;
//# sourceMappingURL=check-for-project-id.js.map