"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const alwayscli_1 = require("@alwaysai/alwayscli");
const app_json_file_1 = require("../../util/app-json-file");
const yes_cli_input_1 = require("../../cli-inputs/yes-cli-input");
const app_install_component_1 = require("../../components/app-install-component");
const environment_1 = require("../../environment");
const check_for_project_id_1 = require("../../components/projects-and-teams/check-for-project-id");
exports.appInstallCliLeaf = alwayscli_1.CliLeaf({
    name: 'install',
    description: "Install this application's dependencies",
    hidden: !Boolean(environment_1.ALWAYSAI_HOME) && !environment_1.ALWAYSAI_OS_PLATFORM,
    namedInputs: {
        yes: yes_cli_input_1.yesCliInput,
    },
    async action(_, { yes }) {
        const appJsonFile = app_json_file_1.AppJsonFile();
        if (appJsonFile.exists()) {
            // if it exits check if it has a project id
            if (appJsonFile.read().project) {
                const contributor = await check_for_project_id_1.checkForProjectId();
                if (!contributor) {
                    throw new alwayscli_1.CliTerseError('You are not part of the team. Make sure to add the project owner to add you.');
                }
                // if this happens dont init the remote step
                // check that the project id belongs to user
            }
            else {
                console.log('No project id found in app.json');
                const selectedProject = await check_for_project_id_1.selectFromListOfProjects();
                appJsonFile.addProjectId(selectedProject['id'].toString());
                // do this part first
                // case where the app id does not exist in a json fille
                // all that needs to be done is add a project id
            }
        }
        await app_install_component_1.appInstallComponent({ yes });
    },
});
//# sourceMappingURL=install.js.map