"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const alwayscli_1 = require("@alwaysai/alwayscli");
const app_json_file_1 = require("../../util/app-json-file");
const app_start_component_1 = require("../../components/app-start-component");
const check_for_project_id_1 = require("../../components/projects-and-teams/check-for-project-id");
// Currently we run the "start" command as a superuser because otherwise we
// cannot connect to the NCS accelerator. It ~should~ be possible to connect to
// the NCS even without --privileged mode, but we need to do the work to set
// that up. See https://movidius.github.io/ncsdk/docker.html
exports.appStartCliLeaf = alwayscli_1.CliLeaf({
    name: 'start',
    namedInputs: {
        'no-superuser': alwayscli_1.CliFlagInput({
            description: 'If running in a container, do so as the login user instead of as the superuser "root"',
        }),
    },
    escapedInput: alwayscli_1.CliStringArrayInput({
        placeholder: '<args>',
        description: 'Arguments passed directly to the application',
    }),
    description: 'Run this application\'s "start" script',
    async action(_, opts, escaped) {
        const appJsonFile = app_json_file_1.AppJsonFile();
        if (appJsonFile.exists()) {
            // if it exits check if it has a project id
            if (appJsonFile.read().project) {
                const contributor = await check_for_project_id_1.checkForProjectId();
                if (!contributor) {
                    throw new alwayscli_1.CliTerseError('You are not part of the team. Make sure to add the project owner to add you.');
                }
                // if this happens dont init the remote step
                // check that the project id belongs to user
            }
            else {
                console.log('No project id found in app.json');
                const selectedProject = await check_for_project_id_1.selectFromListOfProjects();
                appJsonFile.addProjectId(selectedProject['id'].toString());
                // do this part first
                // case where the app id does not exist in a json fille
                // all that needs to be done is add a project id
            }
        }
        const exitCode = await app_start_component_1.appStartComponent({
            noSuperuser: opts['no-superuser'],
            args: escaped,
        });
        process.exit(exitCode);
    },
});
//# sourceMappingURL=start.js.map