"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require('fs');
const path = require('path');
const parseStringSync = require('xml2js-parser').parseStringSync;
const constants_1 = require("./constants");
const DIRECTORY_STRUCTURE = `
alwaysAI retraining expects a PascalVOC datset. The zipped directory must have Pascal VOC
formatted annotation files with a "${constants_1.ImageDirName}" directory containing the images corresponding to the annotation
files found in "${constants_1.AnnotationDirName}"

- ${constants_1.AnnotationDirName}
    - annotation xml or json files
- ${constants_1.ImageDirName}
    - corresponding image files in jpeg format (.jpg or .jpeg)
`;
function validateDataset(datasetDir) {
    const datasetFilesNames = fs.readdirSync(datasetDir);
    if (!datasetFilesNames.includes(constants_1.AnnotationDirName)) {
        throw new Error(`${constants_1.AnnotationDirName} directory not found ${DIRECTORY_STRUCTURE}`);
    }
    validateAnnotationsDirectory(path.join(datasetDir, constants_1.AnnotationDirName));
    if (!datasetFilesNames.includes(constants_1.ImageDirName)) {
        throw new Error(`${constants_1.ImageDirName} directory not found ${DIRECTORY_STRUCTURE}`);
    }
    validateImageDirectory(path.join(datasetDir, constants_1.ImageDirName));
}
exports.validateDataset = validateDataset;
function validateAnnotationsDirectory(dir) {
    const annotationFileNames = fs.readdirSync(dir);
    if (annotationFileNames.length === 0) {
        throw new Error(`${constants_1.AnnotationDirName} directory empty ${DIRECTORY_STRUCTURE}`);
    }
    let numXmlFiles = 0;
    annotationFileNames.forEach((annotationFileName) => {
        if (annotationFileName.toLowerCase().endsWith('.xml')) {
            try {
                parseStringSync(fs.readFileSync(path.join(dir, annotationFileName)));
            }
            catch (e) {
                throw new Error(`Failed to parse ${annotationFileName}, The xml file might be incorrectly formatted. ${DIRECTORY_STRUCTURE} ${e.message}`);
            }
            numXmlFiles += 1;
        }
    });
    if (numXmlFiles === 0) {
        throw new Error(`No xml files in ${constants_1.AnnotationDirName} directory ${DIRECTORY_STRUCTURE}`);
    }
}
function validateImageDirectory(dir) {
    const imageFileNames = fs.readdirSync(dir);
    if (imageFileNames.length === 0) {
        throw new Error(`${constants_1.ImageDirName} directory empty ${DIRECTORY_STRUCTURE}`);
    }
    let jpgFiles = 0;
    imageFileNames.forEach((imageFileName) => {
        if (imageFileName.toLowerCase().endsWith('.jpg') ||
            imageFileName.toLowerCase().endsWith('.jpeg')) {
            jpgFiles += 1;
        }
    });
    if (jpgFiles === 0) {
        throw new Error(`No jpg or jpeg files in ${constants_1.ImageDirName} directory ${DIRECTORY_STRUCTURE}`);
    }
}
//# sourceMappingURL=validate-dataset.js.map