"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const alwayscli_1 = require("@alwaysai/alwayscli");
const yes_cli_input_1 = require("../cli-inputs/yes-cli-input");
const check_for_project_id_1 = require("../components/projects-and-teams/check-for-project-id");
const app_configure_component_1 = require("../components/app-configure-component");
const target_protocol_cli_input_1 = require("../cli-inputs/target-protocol-cli-input");
const target_hostname_cli_input_1 = require("../cli-inputs/target-hostname-cli-input");
const target_path_cli_input_1 = require("../cli-inputs/target-path-cli-input");
const not_allowed_with_message_1 = require("../util/not-allowed-with-message");
const target_protocol_1 = require("../util/target-protocol");
const environment_1 = require("../environment");
exports.test2 = alwayscli_1.CliLeaf({
    name: 'init',
    description: 'Initiate a alwaysai project',
    namedInputs: {
        use: alwayscli_1.CliStringInput({
            description: 'Pick a project name',
        }),
        yes: yes_cli_input_1.yesCliInput,
        protocol: target_protocol_cli_input_1.targetProtocolCliInput,
        hostname: target_hostname_cli_input_1.targetHostnameCliInput,
        path: target_path_cli_input_1.targetPathCliInput,
    },
    async action(_, opts) {
        try {
            const { yes, hostname, path, protocol, use } = opts;
            // Preliminary checks that don't help us with type narrowing
            if (protocol === 'docker:' && environment_1.ALWAYSAI_OS_PLATFORM !== 'linux') {
                throw new alwayscli_1.CliUsageError(`Option "protocol" is not allowed to have value "${target_protocol_1.TargetProtocol['docker:']}" if your operating system platform is "${environment_1.ALWAYSAI_OS_PLATFORM}"`);
            }
            if (protocol === 'docker:' && hostname) {
                throw new alwayscli_1.CliUsageError(not_allowed_with_message_1.NotAllowedWithMessage('hostname', 'protocol', target_protocol_1.TargetProtocol['docker:']));
            }
            if (protocol === 'docker:' && path) {
                throw new alwayscli_1.CliUsageError(not_allowed_with_message_1.NotAllowedWithMessage('path', 'protocol', target_protocol_1.TargetProtocol['docker:']));
            }
            return await app_configure_component_1.appConfigureComponent({
                use,
                yes,
                targetProtocol: protocol,
                targetHostname: hostname,
                targetPath: path,
            });
        }
        catch (exception) { }
    },
});
exports.test3 = alwayscli_1.CliLeaf({
    name: 'test',
    description: 'Initiate a alwaysai project',
    async action(_, opts) {
        try {
            return await check_for_project_id_1.storeDeviceId();
        }
        catch (exception) { }
    },
});
exports.project = alwayscli_1.CliBranch({
    name: 'project',
    description: 'Develop an alwaysAI application',
    subcommands: [exports.test2, exports.test3],
});
//# sourceMappingURL=project.js.map