import { AppJsonFile } from '../util/app-json-file';
import { APP_JSON_FILE_NAME } from '../constants';
import { confirmWriteFilePromptComponent } from './confirm-write-file-prompt-component';
import { findOrWriteAppPyFileComponent } from './find-or-write-app-py-file-component';
import { CliTerseError } from '@alwaysai/alwayscli';
import { UnableToProceedWithoutMessage } from '../util/unable-to-proceed-without-message';
import { Spinner } from '../util/spinner';
import {
  selectTypeOfProject,
  selectFromListOfProjects,
  readJson,
  downloadSingleApp,
  checkForProjectId,
  getListOfProjects,
} from '../components/projects-and-teams/check-for-project-id';
import { cd } from '../util/cd';
import { JsSpawner } from '../util/spawner/js-spawner';

const WRITE_MESSAGE = `Write ${APP_JSON_FILE_NAME}`;
const FOUND_MESSAGE = `Found ${APP_JSON_FILE_NAME}`;

export async function findOrWriteAppJsonFileComponent(props: {
  yes: boolean;
  use?: string;
}) {
  const { yes, use } = props;

  const appJsonFile = AppJsonFile();

  if (appJsonFile.exists()) {
    // if it exits check if it has a project id
    if (appJsonFile.read().project) {
      const contributor = await checkForProjectId();
      if (!contributor) {
        throw new CliTerseError(
          'You are not part of the team. Make sure to add the project owner to add you.',
        );
      }

      // if this happens dont init the remote step
      // check that the project id belongs to user
    } else {
      console.log('No project id found in app.json');
      const selectedProject = await selectFromListOfProjects();
      appJsonFile.addProjectId(selectedProject['id'].toString());

      // do this part first
      // case where the app id does not exist in a json fille
      // all that needs to be done is add a project id
    }
  } else {
    let selectedProject;
    if (use) {
      const listOfProjects = await getListOfProjects();
      const hasProject = listOfProjects.filter(project => project['name'] === use);

      if (hasProject.length) {
        selectedProject = hasProject[0];
      } else {
        selectedProject = await selectFromListOfProjects();
      }
    } else {
      selectedProject = await selectFromListOfProjects();
    }
    // pick which project type you need
    // for this example choose blank
    // blank will keep same flow
    // starter apps already have own json so no need to create
    // so just divide these steps into another if and else
    // read the project and check what kind of starter app it has if its empty then blank

    if (selectedProject.starter_app) {
      console.log(selectedProject.starter_app);
      await downloadSingleApp(process.cwd(), selectedProject.starter_app);

      appJsonFile.addProjectId(selectedProject['id'].toString());
    } else if (selectedProject.git_url) {
      const spawner = JsSpawner();

      spawner.runForegroundSync({
        exe: 'git',
        args: ['clone', selectedProject.git_url],
      });
    } else {
      const projectType = await selectTypeOfProject();
      if (projectType.projects === 'Start Blank') {
        const confirmed =
          yes ||
          (await confirmWriteFilePromptComponent({
            fileName: APP_JSON_FILE_NAME,
            description: 'Configuration file',
          }));
        if (!confirmed) {
          throw new CliTerseError(UnableToProceedWithoutMessage(APP_JSON_FILE_NAME));
        }
        try {
          appJsonFile.initialize();
          appJsonFile.addProjectId(selectedProject['id'].toString());
          Spinner(WRITE_MESSAGE).succeed();
        } catch (exception) {
          Spinner(WRITE_MESSAGE).fail();
          throw exception;
        }
        await findOrWriteAppPyFileComponent({ yes });
      } else if (projectType.projects === 'Use starter app') {
        const pickedStarterApp = await readJson();
        await downloadSingleApp(process.cwd(), pickedStarterApp);

        appJsonFile.addProjectId(selectedProject['id'].toString());
      }
    }
  }
}
