import { CliLeaf, CliTerseError } from '@alwaysai/alwayscli';
import { AppJsonFile } from '../../util/app-json-file';

import { yesCliInput } from '../../cli-inputs/yes-cli-input';
import { appInstallComponent } from '../../components/app-install-component';
import { ALWAYSAI_HOME, ALWAYSAI_OS_PLATFORM } from '../../environment';
import {
  selectTypeOfProject,
  selectFromListOfProjects,
  readJson,
  downloadSingleApp,
  checkForProjectId,
  getListOfProjects,
} from '../../components/projects-and-teams/check-for-project-id';

export const appInstallCliLeaf = CliLeaf({
  name: 'install',
  description: "Install this application's dependencies",
  hidden: !Boolean(ALWAYSAI_HOME) && !ALWAYSAI_OS_PLATFORM,
  namedInputs: {
    yes: yesCliInput,
  },
  async action(_, { yes }) {
    const appJsonFile = AppJsonFile();

    if (appJsonFile.exists()) {
      // if it exits check if it has a project id

      if (appJsonFile.read().project) {
        const contributor = await checkForProjectId();

        if (!contributor) {
          throw new CliTerseError(
            'You are not part of the team. Make sure to add the project owner to add you.',
          );
        }

        // if this happens dont init the remote step
        // check that the project id belongs to user
      } else {
        console.log('No project id found in app.json');
        const selectedProject = await selectFromListOfProjects();
        appJsonFile.addProjectId(selectedProject['id'].toString());

        // do this part first
        // case where the app id does not exist in a json fille
        // all that needs to be done is add a project id
      }
    }
    await appInstallComponent({ yes });
  },
});
