import {
  CliLeaf,
  CliBranch,
  CliStringArrayInput,
  CliStringInput,
  CliUsageError,
} from '@alwaysai/alwayscli';
import { getStarterApps } from '../util/get-starter-apps';
import { yesCliInput } from '../cli-inputs/yes-cli-input';
import { Spinner } from '../util/spinner';
import { ALWAYSAI_STARTER_APPS } from '../constants';
import {
  checkForProjectId,
  selectFromListOfProjects,
  localProject,
  readJson,
  downloadSingleApp,
  newConfig,
  storeDeviceId,
} from '../components/projects-and-teams/check-for-project-id';

import { appConfigureComponent } from '../components/app-configure-component';
import { targetProtocolCliInput } from '../cli-inputs/target-protocol-cli-input';
import { targetHostnameCliInput } from '../cli-inputs/target-hostname-cli-input';
import { targetPathCliInput } from '../cli-inputs/target-path-cli-input';
import { NotAllowedWithMessage } from '../util/not-allowed-with-message';
import { TargetProtocol } from '../util/target-protocol';
import { ALWAYSAI_OS_PLATFORM } from '../environment';
export const test2 = CliLeaf({
  name: 'init',
  description: 'Initiate a alwaysai project',
  namedInputs: {
    use: CliStringInput({
      description: 'Pick a project name',
    }),
    yes: yesCliInput,
    protocol: targetProtocolCliInput,
    hostname: targetHostnameCliInput,
    path: targetPathCliInput,
  },

  async action(_, opts) {
    try {
      const { yes, hostname, path, protocol, use } = opts;

      // Preliminary checks that don't help us with type narrowing
      if (protocol === 'docker:' && ALWAYSAI_OS_PLATFORM !== 'linux') {
        throw new CliUsageError(
          `Option "protocol" is not allowed to have value "${
            TargetProtocol['docker:']
          }" if your operating system platform is "${ALWAYSAI_OS_PLATFORM}"`,
        );
      }

      if (protocol === 'docker:' && hostname) {
        throw new CliUsageError(
          NotAllowedWithMessage('hostname', 'protocol', TargetProtocol['docker:']),
        );
      }

      if (protocol === 'docker:' && path) {
        throw new CliUsageError(
          NotAllowedWithMessage('path', 'protocol', TargetProtocol['docker:']),
        );
      }

      return await appConfigureComponent({
        use,
        yes,
        targetProtocol: protocol,
        targetHostname: hostname,
        targetPath: path,
      });
    } catch (exception) {}
  },
});

export const test3 = CliLeaf({
  name: 'test',
  description: 'Initiate a alwaysai project',

  async action(_, opts) {
    try {
      return await storeDeviceId();
    } catch (exception) {}
  },
});
export const project = CliBranch({
  name: 'project',
  description: 'Develop an alwaysAI application',
  subcommands: [test2, test3],
});
